﻿using System;
using System.Collections.Generic;
using System.IO;
using Fleck;

namespace FleckTest
{
    public class Program
    {
        private static List<IWebSocketConnection> sockets = new List<IWebSocketConnection>();

        private static void Main(string[] args)
        {
            var server = new WebSocketServer("ws://localhost:8181");
            server.Start(socket =>
            {
                socket.OnOpen = () => OnOpen(socket);
                socket.OnClose = () => OnClose(socket);
                socket.OnMessage = m => OnMessage(socket, m);
            });
            string consoleReadLine;
            do
            {
                consoleReadLine = Console.ReadLine();
                sockets.ForEach(s => s.Send(consoleReadLine));
                if (consoleReadLine == "perf")
                {
                    var start = DateTime.Now;
                    for (int i = 0; i < 1000; i++)
                    {
                        sockets[0].Send("performaceTest"+i); 
                    }
                    Console.WriteLine(string.Format("Time: {0}", DateTime.Now - start));
                }
                if (consoleReadLine == "len")
                {
                    var content = File.ReadAllText("testFile.dll");
                    var start = DateTime.Now;
                    sockets[0].Send(content);
                    Console.WriteLine(string.Format("Time: {0}", DateTime.Now - start));
                }
            } while (consoleReadLine != "exit");
        }

        private static void OnClose(IWebSocketConnection socket)
        {
            Console.WriteLine(string.Format("Connection with {0} Close!", socket.ConnectionInfo.ClientIpAddress));
        }

        private static void OnMessage(IWebSocketConnection socket, string message)
        {
            socket.Send(message);
            Console.WriteLine("Client " + socket.ConnectionInfo.ClientIpAddress + " sended: " + message);
        }

        private static void OnOpen(IWebSocketConnection socket)
        {
            Console.WriteLine(string.Format("Connection with {0} Open!", socket.ConnectionInfo.ClientIpAddress));
            sockets.Add(socket);
        }
    }
}